C========================================================================
C  Program PLOTTER
C          ~~~~~~~
C
C DESCRIPTION:
C
C   This program was designed to produce plots on any of the 
C   following devices: APPLICON, VERSATEC, or OPTRONICS plotters, or
C   Tektronix 41xx graphic display terminals.
C
C   The plots can be any size and can have a user-defined color scheme.
C
C   If a Applicon, Versatec, or Optronics plot is to be created,
C   the program produces a file called UNIRAST. The UNIRAST file must
C   then be sent to the plotter uwith an appropriate device driver program.
C
C
C INPUT: 
C        Accepts files in Morpholog format, in the SPANS Browse (.PIC
C        = HALO .CUT) run-length encoded format, or PCI's BSQ (.PIP)
C        channel format. The .PIC or BSQ files must
C        be uploaded from a PC in such a way that 512-byte VAX records
C        are created. As of Aug. 1989, the only way of doing this
C        is using the TCP/IP protocol - PROCOMM and CrossTalk don't
C        do it right even when you specify the transfer as BINARY. 
C        The .PIC files are decoded by the subroutine Decode_PIC. 
C
C        Palettes for color class data can be either in an ASCII .PAL
C        file, or else in a SPANS .BIC file that was uploaded from a PC in 
C        the same way as .PIC files. The .BIC files are decoded by the
C        subroutine Decode_BIC.
C
C IMPLEMENTATION DETAILS:
C
C  1)    The UNIRAS/GIMAGE package, which is used for plotting, scans
C        along what it considers to be the Y-axis, which turns out
C        to be horizontal strokes on the plotting paper. Since BSQ and
C        PIC files arrive in row-order and not column order, this
C        is the result:
C
C                    plotting direction
C
C                            :
C                            :
C                            V
C                  ______
C        +-----~~~~      ~~~~~-----____     ____-----+
C        |                             ~~~~~         |
C        |    y --->                                 |
C        | x                                         |
C        | |                                         |
C        | V   +                                  +  |
C        |       +------------------+                |
C        |       | top of image     | legend         |
C        |       |                  |                |
C        |       |                  | north arrow    |
C        |       |                  | scale          |
C        |       |                  | comments       |
C        |       +------------------+                |
C        |             titles                        |
C        |     +                                  +  |
C        |         ______                            |
C        +-----~~~~      ~~~~~-----____     ____-----+
C                                      ~~~~~
C
C        Therefore, X coordinates (xorig, ixsiz, etc.) used in
C        this program refer to ROWS of the INPUT image, and 
C        Y coordinates refer to COLUMNS of the INPUT image.
C
C  2)    The following features of PLOTTER exist in the code but have
C        been commented out because of a lack of urgency or time to
C        debug them:
C
C                   i) ability to plot SPANS .RNL files
C
C                  ii) doing black and white 16-pattern plots
C
C                 iii) using 1-3 lookup tables for channel file data
C
C                  iv) using default UNIRAS colour table for theme
C                      data, instead of loading new palette
C
C        
C COMPILE/LINK PROGRAM: 
C
C      $ FOR  PLOTTER
C      $ LINK PLOTTER,PLOTROUT.OBJ,LEGMAKER.OBJ,
C             DECODE.OBJ,UNILIB/LIB,VERLIB/LIB
C
C                     - or -
C
C       $ FOR PLOTTER
C       $ @linkplot
C
C AUTHOR: Nadia Shewchenko, Mark Tooker
C         Geological Survey of Canada
C
C DATE: Sept., 1989
C
C ************************************************************************

      PROGRAM plot_facility

C*************************************************************************
C              Program storage
C*************************************************************************

      integer   plotter,lastplotter ! 1 = Applicon; 2 = Versatec; 3 = Optronics
C                                   ! 4 = MT41xx; 5 = select from GROUTE

      integer   data_type       ! 1 = 3 channels; 2 = color classes

      integer   file_type       ! 1 = Morpholog row-by-row
C                               ! 2 = PCI .BSQ file
C                               ! 3 = SPANS 4.0 .PIC file
C                               ! 4 = SPANS RNL 8-bit not encoded

      integer*4 ncols,nrows     ! number of rows, columns in input image
      integer*4 ixsiz,iysiz     ! # of pixels to be plotted in each column/row
      integer*4 lut(3,0:255)    ! channel files look up table
      integer*4 index(4096)     ! shows which pixels to decimate/repeat
      integer*4 kcolours        ! maximum # colors in palette
      integer*2 TextColor       ! the value class 255 should be changed to
      integer*2 BGColor         ! the value class 0 should be changed to
      integer   ctable(256,2)   ! color table
      integer   nthemes         ! # of color themes
      integer   leg_ctable(12,3)! legend color table
      integer   kolor(99)       ! sets up color class limits
      integer   ic(99),im(99),  ! holds color table entries from palette
     +          iy(99)          !
      integer*4 ShortNum        ! num of lines to really plot
      integer*4 nlook           !
      integer   PalChoice       ! palette choice
      integer   PlotCount       ! counts plots for log file

      character*30 dirName,defdir     ! default directory
      character*50 fin, fin3(3)       ! input data file name(s)
      character*50 lutfin3(3)         ! lookup table names
      character key*1                 ! utility input char.
      character title1*41, title2*41  ! plot titles
      character*36 CommentLine(4)
      character*31 leg_title          ! legend title
      character*25 Theme(12)          ! legend theme titles
      character*60 UsageArray(100)    ! used to record paper usage -
C                                     ! 1 character shows approx. 30x30 mm
      character*50 PalName            ! file name of color palette
      character*50 BicName            ! file name of BIC palette
      character*50 BWpal              ! file name of B & W palette
      character*40 PlotterName(5)     ! holds name of each plotter type
      character*20 file_type_name(4)  ! name for each file type

      logical looks    ! true if channel file lut's exist
      logical patterns ! true if B&W patterns wanted
      logical channel3 ! true if 3 channel data
      logical first    ! true when first image plot being done
      logical Is_RGB   ! true if RGB colors; false if CMY
      logical NewTable ! true if new color table; false if standard palette
      logical WantsLegend   ! true if legend wanted
      logical WantsTitle    ! true if titles wanted
      logical WantsComments ! true if comments wanted
      logical WantsRegs     ! true if registration marks wanted
      logical Shorten       ! true if ShortNum lines to be plotted
      logical log           ! true if log file wanted
      logical OK            ! true if subr. completed OK

      real*4 xblk                  ! block size in legend
      real*4 largest_x,largest_y   ! furthest extensions in mm of current plot
      real*4 overall_largest_x,
     +       overall_largest_y     ! furthest extensions of any plot so far
      real*4 c_length              ! lngth of comments
      real*4 leg_length            ! lngth of legend 
      real*4 xtitle                ! height of titles in mm
      real*4 dec                   ! decimation factor (should be = integer 
C                                  ! or = 1/integer)
      real*4 scale                 ! scaling factor of pixels to plotter grid
      real*4 pixsz                 ! size of each pixel in mm
      real*4 xsize,ysize           ! size of plotted image in mm
      real*4 xorig,yorig           ! coordinates in mm of image origin
      real*4 rmax                  ! max. value of channel file data
      real   zclass(98)            ! color classes (for GIMZCL)
      real*4 map_scale             ! # of meters = 1/3 width of image
      real*4 orient                ! heading of north arrow in degrees
      real   PlotWidth,PlotHeight  ! size of an image plot

      COMMON /GEO6XX/ KDUMMY,KWORK(176000)    ! ?? needed by UNIRAS?

      include '[bcvax.fort]txtfnct.cns'   ! include text constants
 
C **************************************************************************
C      INITIALIZE DATA
C **************************************************************************

      file_type_name(1) = 'Morpholog'
      file_type_name(2) = 'PCI .BSQ'
      file_type_name(3) = 'SPANS 4.0 .PIC'
      file_type_name(4) = 'SPANS RNL 8-bit not encoded'

      PlotterName(1) = 'Applicon'
      PlotterName(2) = 'Versatec'
      PlotterName(3) = 'Optronics'
      PlotterName(4) = 'Tektronix 41**'
      PlotterName(5) = '(To be selected from GROUTE)'

      plotCount = 1
      data_type = 1
      nchan = 3
      nlook = 2
      rmax = 255
      plotter = 2
      lastplotter = 0
      PalChoice = 1
      DirName = '                              '
      first = .TRUE.
      overall_largest_x = 0.0
      overall_largest_y = 0.0  
      xorig = 0.0
      yorig = 0.0

C **************************************************************************
C      begin main body of PLOTTER program
C **************************************************************************

C................................................
C.....loop from here for every image plotted.....

999   largest_x = 0.0
      largest_y = 0.0

      write (*,*) SBLACK,CLR,' '
      write (*,*) DBLW,UL,'File Plotting Facility',RST
      write (*,*)
      print *,' '

      print *,' '
      print *,'Please enter the following information'
      print *,' '

      if (First) then

C........log file query.....

30       write (*,40)
40       format (' LOG FILE:',/,
     +           ' Do you want a log file to be kept? (Y/N) [N]: ',$)
         read (*,'(a1)') key

         if (lchar(key).eq.0) then
            key = 'N'
         end if

         if ((key.eq.'Y').or.(key.eq.'y')) then
            log = .true.
            open (unit=66,file='PLOTTER.LOG',form='formatted',
     +            status='new')
            write (66,*) 'PLOTTER PROGRAM LOG FILE'
            write (66,*) '~~~~~~~~~~~~~~~~~~~~~~~~'
         else
            log = .false.
         end if

50       write (*,60)
60       format (/,' DIRECTORY:',/,
     +'    Enter the device and/or directory where data files ',/,
     +'    are located (Hit <Enter> for default directory): ',$)
         read (*,'(a30)',err=50) DirName

         if ((log).and.(lchar(DirName).ne.0)) then
            write (66,*) 'DEFAULT DIRECTORY FOR DATA FILES: ',DirName
         end if

      end if

C ********************************************************************
C           QUERY ON HOW DATA ORGANIZED 
C ********************************************************************

      write(*,*) SBLACK,CLR,' '     
      write(*,*) DBLW,UL,'Data Organization',RST

100   write (*,200) 
200   format(/,
     +' (1) Data is organized in 1-3 files, one for each channel ',/,
     +' (2) Data in 1 file, and consists of color class numbers',/)
      call ENTINT (' ENTER Choice (1-2) ',.true.,
     +                          data_type,data_type,OK)

      if (((data_type.lt.1).or.(data_type.gt.2)).or.(.not.OK)) then
         print *,'**ERROR** Invalid choice!'
         go to 100
      endif

      if (log) then
         write (66,*) 'PLOT :',PlotCount
         write (66,*) '~~~~~~~~~~~~~~~~~'
         if (data_type.eq.1) then
             write (66,*) 'DATA TYPE: 1  (channel files)'
         else
             write (66,*) 'DATA TYPE: 2  (color class numbers)'
         end if
      end if

C ********************************************************************
C          GET FILE NAMES OF ALL THE NEEDED DATA AND LOOKUP FILES
C ********************************************************************

      IF (data_type.eq.1) THEN
         channel3 = .TRUE.
         call Channel_Files_Queries
     +                     (nchan,nlook,lut,lutfin3,ncols,
     +                      nrows,looks,fin3,First,
     +                      DirName,file_type,file_type_name)

         if (log) then
            write (66,*) 'FILE TYPE: ',file_type,' (',
     +  file_type_name(file_type)(1:lchar(file_type_name(file_type))),
     +  ')'
            write (66,*) 'NUMBER OF CHANNELS: ',nchan
            do i = 1,nchan
               write (66,*) '   FILE ',i,': ',fin3(i)
            end do
            write (66,*) '   # ROWS: ',nrows,'    # COLUMNS: ',ncols
         end if
      ELSE
         channel3 = .FALSE.
         call Class_File_Queries(ncols,nrows,fin,First,
     +                           DirName,file_type,file_type_name)

         if (log) then
            write (66,*) 'FILE TYPE: ',file_type,' (',
     +  file_type_name(file_type)(1:lchar(file_type_name(file_type))),
     +  ')'
            write (66,*) '   DATA FILE: ',fin
            write (66,*) '   # ROWS: ',nrows,'   # COLUMNS: ',ncols
         end if
      ENDIF

      print *,' '
      print *,'Press ENTER to continue...'
      read(*,'(a1)') key

C ********************************************************************
C            GET PLOTTER AND PLOT SIZE
C ********************************************************************

      patterns = .FALSE.
 
      write(*,*) SBLACK,CLR,' '     
      write(*,*) DBLW,UL,'Plotter Selection',RST

      call Get_Plotter(plotter,lastplotter,plpix,first)

      if (log) then
         write (66,*) 'PLOTTER: ',plotter,'  (',PlotterName(plotter)
     +             (1:lchar(plotterName(plotter))), ')'
      end if

      scale = 1.0
      dec = 1.0
      call Get_Values (plotter,plpix,nrows,ncols,ixsiz,iysiz,
     +                  xsize,ysize,dec,scale,first,PlotterName)

      if (log) then
         write (66,*) 'SCALING FACTOR: ',scale
         write (66,*) '   SIZE IN X-DIRECTION: ',xsize
         write (66,*) '   SIZE IN Y-DIRECTION: ',ysize
      end if

C ********************************************************************
C           Set up color palettes, either by using channels or
C              by picking colors and color boundaries.
C ********************************************************************

      write (*,*) SBLACK,CLR,' '
      write(*,*) DBLW,UL,'Color Scheme:',RST
      print *,' '

      call Color_Query(patterns,Is_RGB,nlook,data_type,First)

      if (log) then
         if (Is_RGB) then
            write (66,*) 'COLOR SCHEME: RGB'
         else
            write (66,*) 'COLOR SCHEME: CMY'
         end if
      end if

      if (data_type.eq.1) then
         write (*,*) SBLACK,CLR,' '
         write (*,*) DBLW,UL,'Colour Value Range',RST

         call Set_Up_Channels_Query (rmax)

         if (log) then
            write (66,*) '   MAXIMUM COLOR CHANNEL VALUE: ',rmax
         end if

      else
         write (*,*) SBLACK,CLR,' '
         write(*,*) DBLW,UL,'Palette Selection:',RST
         print *,' '

         call Set_Up_Color_Table_Queries
     +                       (patterns,ic,im,iy,
     +                       zclass,kolor,TextColor,BGColor,
     +                       Is_RGB,NewTable,First,PalChoice,
     +                       fin,PalName,BicName,BWpal,DirName)

         if (log) then
            write (66,*) '    PALETTE FILE ID: ',PalName
         end if

      end if

C ********************************************************************
C              Determine what 'extras' user wants 
C                       added to plot
C ******************************************************************** 

      write (*,*) SBLACK,CLR,' '
      write (*,*) DBLW,UL,'Annotations',RST

      call Get_Legend2 (WantsLegend,leg_title,nthemes,theme,
     +                   map_scale,orient,leg_ctable,Is_RGB,
     +                   data_type,ysize,first)

      if (log) then
         if (WantsLegend) then
            write (66,*) 'LEGEND TITLE: ',leg_title
            write (66,*) '   NUMBER OF THEMES: ',nthemes
            do i = 1,nthemes
               write (66,*) '       THEME ',i,': ',
     +              leg_ctable(1,i),leg_ctable(2,i),leg_ctable(3,i),
     +              '  TITLE: ',theme(i)
            end do
            write (66,*) '   NORTH ORIENTATION: ',orient
            write (66,*) '   MAP WIDTH (FOR SCALE): ',map_scale
         else
            write (66,*) 'NO LEGEND'
         end if
      end if

      call Get_Titles(title1,title2,WantsTitle,first,log)

      if (log) then
         if (WantsTitle) then
            write (66,*) 'TITLE 1: ',title1
            write (66,*) 'TITLE 2: ',title2
         else
            write (66,*) 'NO TITLES'
         end if
      end if

      call Get_Comments (CommentLine,WantsComments,first,log)

      if (log) then
         if (WantsComments) then 
            do i = 1,4
               write (66,*) 'COMMENT ',i,': ',CommentLine(i)
            end do
         else
            write (66,*) 'NO COMMENTS'
         end if
      end if

      call Register_Query (WantsRegs,first,log)

      if (log) then
         if (WantsRegs) then
            write (66,*) 'INCLUDE REGISTRATION MARKS'
         else
            write (66,*) 'NO REGISTRATION MARKS'
         end if
      end if

C ********************************************************************
C                  Find out the origin at which to plot
C ********************************************************************

      write(*,*) SBLACK,CLR,' '
      write (*,*) DBLW,UL,'Origin',RST

      call Get_Origin (xorig,yorig,xsize,ysize,first,
     +                 overall_largest_x,overall_largest_y,
     +                 UsageArray,plotter,lastplotter,WantsRegs)

      if (log) then
         write (66,*) 'X-ORIGIN: ',xorig
         write (66,*) 'Y-ORIGIN: ',yorig
      end if

      largest_x = AMAX1 (largest_x, xorig+xsize)
      largest_y = AMAX1 (largest_y, yorig+ysize)

C ********************************************************************
C               Find out if short plot wanted
C ********************************************************************

      write(*,*) SBLACK,CLR,' '
      write (*,*) DBLW,UL,'Abbreviation',RST

      call Shorten_Query (Shorten,ShortNum)

      if (log) then
         if (Shorten) then
            write (66,*) 'SHORTENED # LINES TO PLOT: ',ShortNum
         else
            write (66,*) 'NO SHORTENING OF PLOT'
         end if
      end if

      print *,' '
      print *,'Press ENTER to continue'
      read(*,'(a1)') key

C ********************************************************************
C                     Get device ready to plot
C ********************************************************************

      write (*,*) SBLACK,CLR,' '
      write (*,*) DBLW,UL,'Initializing Graphics',RST

      call Initialize_Graphics(plotter,lastplotter,plpix,xorig,yorig)

      print *,' '
      print *,'Press ENTER to continue'
      read(*,'(a1)') key

C ********************************************************************
C               Plot titles, legend, comments, etc.
C ********************************************************************

      if ((Wantslegend.or.WantsTitle.or.WantsComments.or.
     +    WantsRegs).and.(plotter.ne.4))
     +then
         write(*,*) SBLACK,CLR,' '
         write (*,*) DBLW,UL,'Plotting Annotations',RST
      end if

      if (WantsLegend) then
         if (plotter.ne.4) then
            print *,' '
            print *,'Plotting legend...'
         end if
         call plot_legend2 (leg_title,nthemes,theme,map_scale,orient,
     +                      leg_ctable,xorig,yorig,xsize,ysize,
     +                      Is_RGB,plotter,leg_length)

         largest_y = AMAX1 (largest_y,(yorig+ysize+leg_length))
      end if

      if (WantsTitle) then
         if (plotter.ne.4) then
            print *,' '
            print *,'Plotting titles...'
         end if
         call Write_Titles(title1,title2,plpix,xsize,ysize,
     +                     xorig,yorig,xtitle,plotter)
         largest_x = AMAX1 (largest_x,(xorig+xsize+xtitle))
      endif

      if (WantsComments) then
         if (plotter.ne.4) then
            print *,' '
            print *,'Plotting comments...'
         end if
         call Write_Comments(CommentLine,xsize,ysize,
     +                       xorig,yorig,c_length,plotter)

         largest_y = AMAX1 (largest_y,(yorig+ysize+c_length))
      endif

      if (WantsRegs) then
         if (plotter.ne.4) then
            print *,' '
            print *,'Plotting Registration marks...'
         end if

         call Plot_Registration (xorig,yorig,largest_x,
     +                           largest_y,plpix,plotter,Is_RGB)

C........note that largest_x and largest_y are updated.....
C........by the preceding subroutine.....

      end if

      if ((Wantslegend.or.WantsTitle.or.WantsComments.or.
     +    WantsRegs).and.(plotter.ne.4))
     +then
         print *,' '
         print *,'Press ENTER to continue'
         read(*,'(a1)') key
      end if

C ********************************************************************
C                            Plot image
C ********************************************************************

      call Set_Color_Mode (patterns,Is_RGB)

      if (data_type.eq.1) then
           call Set_Up_Channels (rmax)

           if (plotter.ne.4) then
              write (*,*) SBLACK,CLR,' '
              write(*,*) DBLW,UL,'Plotting:',RST
              print *,' '
           end if

           call Plot_Channel_Files(plotter,iysiz,ixsiz,nrows,
     +           ncols,nchan,nlook,looks,lut,plpix,fin3,file_type,
     +           xorig,yorig,Shorten,ShortNum)
      else
           call Set_Up_Color_Table (patterns,NewTable,Is_RGB,
     +                              kolor,zclass,ic,im,iy)

           if (plotter.ne.4) then
              write (*,*) SBLACK,CLR,' '
              write(*,*) DBLW,UL,'Plotting:',RST
              print *,' '
           end if

           call Plot_Class_File(ncols,nrows,plotter,
     +                          plpix,fin,file_type,
     +                          TextColor,BGColor,xorig,yorig,
     +                          Shorten,ShortNum)
      endif


C ********************************************************************
C                 Find out if another plot wanted
C ********************************************************************

      if (plotter.eq.4) then
         print *,'----------------------------------------------'
         print *,'Hit <SHIFT><D Erase> to clear text from image.'
         print *,'Then, hit <ENTER> to continue'
         print *,'----------------------------------------------'
         read (*,'(a1)') key                       ! wait after drawing
      else
         print *,' '
         print *,'Hit <ENTER> to continue'
         read (*,'(a1)') key                       ! wait after drawing
      end if

      write (*,*) SBLACK,CLR,' '
      write(*,*) DBLW,UL,'Finished Plotting',RST
      print *,' '

      if (WantsRegs) then
         PlotWidth  = (largest_y - yorig) + 10
         PlotHeight = (largest_x - xorig) + 10
      else
         PlotWidth  = largest_y - yorig
         PlotHeight = largest_x - xorig
      end if

      print *,'Final size of that plot ',
     +        '(including annotations) in mm : '
      print *,'   Width  = ',PlotWidth
      print *,'   Height = ',PlotHeight

      if (log) then
         write (66,*) 'FINISHED PLOTTING'
         write (66,*) '   WIDTH PLOTTED : ', PlotWidth
         write (66,*) '   HEIGHT PLOTTED: ', PlotHeight
      end if

      overall_largest_x = AMAX1 (overall_largest_x, largest_x)
      overall_largest_y = AMAX1 (overall_largest_y, largest_y)

450   write (*,460)
460   format(//,' Plot another image? (Y/N) [Y]: ',$)
      read(*,'(a1)',err=450,iostat=ierr) key
      if (((key.eq.'Y').or.(key.eq.'y')).or.(lchar(key).eq.0)) then
         first = .FALSE.
         lastplotter = plotter
         PlotCount = PlotCount + 1
         go to 999
      endif

      print *,' '
      print *,'******* END OF PLOTTER PROGRAM *******'
      print *,' '

      if (plotter.ne.4) then
         print *,' '
         print *,'Overall size of plot in mm : '
         print *,'Width  = ',(overall_largest_y),
     +           'Height = ',(overall_largest_x)
         print *,' '
         print *,'Plot file has been generated with logical name ',
     +           'UNIRAST'
      end if

      if (log) then
         write (66,*) '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~'
         write (66,*) 'FINISHED PLOTTER PROGRAM'
         write (66,*) 'OVERALL PLOT SIZE'
         write (66,*) '   WIDTH : ',overall_largest_y
         write (66,*) '   HEIGHT: ',overall_largest_x
         if (plotter.ne.4) then
            write (66,*) 'LOGICAL NAME OF PLOT FILE: UNIRAST'
         end if
         close (66)
      end if

      stop
      end

C *****************************************************************
C    SUBROUTINE : Channel_Files_Queries
C
C    PURPOSE: This routine queries the user for the channel file 
C             names and for the lookup file names is they exist.
C             The number of columns and rows of each of the
C             channel files is read.
C
C    INPUT PARAMETERS:
C             first - true if this is first plot
C             DirName - default directory for data files
C             File_type_name - name of file type
C
C    OUTPUT PARAMETERS:
C             nchan - the number of channel files 
C             nlook - 1 if color plot; 3 if b/w plot
C             lut   - lookup table array
C             ncols - the number of pixels per line
C             nrows - the number of scan lines in the image
C             looks - logical values as to whether lookup table exists or not
C             fin3  - names of three channel files
C             file_type - format of data file(s)
C
C *****************************************************************

      subroutine Channel_Files_Queries
     +                        (nchan,nlook,lut,lutfin3,ncols,
     +                         nrows,looks,fin3,First,
     +                         DirName,file_type,file_type_name)

      integer*4 nchan,nlook,lut(3,0:255),ncols,nrows,cols(3),rows(3)
      integer file_type
      character*50 lutfin3(3),fin3(3),deffin
      character*30 dirName            ! default directory
      character*20 file_type_name(4)
      character key,defkey
      logical looks,First,OK
      logical*1 Byte

1     call ENTINT (' Number of Input Channels (1-3) ',.true.,
     +               nchan,nchan,OK)
      if ((nchan.gt.3.or.nchan.lt.1).or.(.not.OK)) then
         print *,'**ERROR** value out of range'
         print *,' ' 
         go to 1
      endif         

C.....find out if in BSQ or Morpholog format.....

100   if ((First).or.(file_type.eq.2)) then
         iChoice = 2
      else
         iChoice = 1
      end if

      write (*,110) file_type_name(1),file_type_name(2)
110   format (/,' (1) - Files are type ',a20,/,
     +          ' (2) - Files are type ',a20)
      call ENTINT (' Enter Choice (1-2) ',.true.,iChoice,iChoice,OK)
      if (((iChoice.lt.1).or.(iChoice.gt.2)).or.(.not.OK)) then
         goto 100
      else
         file_type = iChoice
      end if
     
c ------ get type of plot 
C
C3     if ((First).or.(nlook.eq.3)) nlook = 2
C      print *,' '
C      print *,'Type of Plot:'
C      write (*,4) 
C4     format(' (1) Black/White Plot' ,
C     +     /,' (2) Colour Plot',/)
C      call ENTINT('ENTER choice (1-2) ',.true.,nlook,nlook,OK)
C
C      if ((nlook.ne.1.and.nlook.ne.2).or.(.not.OK)) then
C         print *, '**ERROR** invalid response!'
C         print *,' '
C         go to 3
C      endif
C
C      if (nlook.eq.1) then
C         print *,' '
C         print *,'*** WARNING *** Not working for black and white.' 
C         print *,' '
C      end if
C
      if (nlook.eq.2) nlook = 3

c -----  query if lookup table
C
C5     if ((First).or.(.not.Looks)) then
C         defkey = 'N'
C      else
C         defkey = 'Y'
C      end if
C
C     write (*,6) defkey 
C6     format( /,' Do Lookup Tables Exist for the Channels? (Y/N) ',
C     +           '[',a1,']: ',$)
C
C      read(*,'(a1)') key
C      if (key.eq.' ') key = defkey

C      if ((key.eq.'y').or.(key.eq.'Y')) then
C         looks = .true.
C         print *,' '
C         print *,'*** WARNING *** Not debugged for lookup tables'
C         print *,' '
C      else
C         looks = .false.
C      end if

      print *,' '
      do  i=1,nchan

         if ((First).or.(lchar(fin3(i)).eq.0)) then
            deffin = '                                '
         else
           deffin = fin3(i)( index(fin3(i),']')+1 : lchar(fin3(i)) )
         end if

77       write  (*,333) i, deffin
333      format (' Input file name for channel',i2,' is ["',
     +            a<lchar(deffin)>,'"]:',$)
          read   (*,'(a32)') fin3(i)

          call Convert_filename (fin3(i), deffin, DirName)

          open   (unit=i,name=fin3(i),status='old',form='unformatted',
     +              err=76)
          go to 78
76        print *,'**ERROR** Cannot open file ', fin3(i)
          go to 77
78        continue

          call Get_File_Size (i,file_type,rows(i),cols(i),first)

C          read(i) rows(i),cols(i)

C........................................................................
Cc -----  query if lookup table
C
C          if     (looks) then
C
C88             if (First) then
C                  deffin = '                                '
C               else
C                  deffin = fin3(i) 
C     +               ( index(fin3(i),']')+1 : lchar(fin3(i)) )
C                  deffin = deffin ( 1 : index(deffin,'.') ) //
C     +               'LUT'
C               end if
C
C               write  (*,633) i,deffin
C633            format (' Name of lookup file for channel ',i2, ' ["',
C     +                 a<lchar(deffin)>,'"]:',$)
C               read   (*,'(a32)') lutfin3(i)
C
C               call convert_filename (lutfin3(i), deffin, DirName)
C
C               open   (unit=i*10,name=lutfin3(i),status='old',
C     +                 form='formatted',err=87)
C               go to 86
C87             print *,'**ERROR** Cannor open file ',lutfin3(i)
C               go to 88
C86             continue
C               if (nchan.ne.1.or.nlook.eq.1) then
C                  do  j=0,255
C                     read  (i*10,10) lut(i,j)
C                  enddo
C               else
C                  do  j=0,255
C                    read  (i*10,10) (lut(ii,j),ii=1,nlook)
C10                  format(4i4)
C                 enddo
C               endif
C          endif
C..........................................................................

      enddo
   
      print *,' '
      do i = 1, nchan
        print 44,i,cols(i),rows(i)
44      format(' File ',i2,' has ',i5,' Columns and ',i5,' Rows.')
      end do
  
      nrows = rows(1)
      ncols = cols(1)
   
      return 
      end

C *****************************************************************
C    SUBROUTINE : Class_File_Queries
C
C    PURPOSE: This routine queries the user for the file containing
C             the colour class values.
C             The number of columns and rows of each of the file is
C             read.
C 
C    INPUT PARAMETERS:
C             first - true if this is the first plot
C             file_type_name - names of file types
C             DirName - name of default directory for data file
C
C    OUTPUT PARAMETERS:
C             ncols - the number of pixels per line
C             nrows - the number of scan lines in the image
C             file_type - format of data file
C             DirName - default directory for data files
C             fin - name of the data file
C
C *****************************************************************

      subroutine Class_File_Queries (ncols,nrows,fin,first,
     +                               DirName,file_type,file_type_name)

C...subroutine parameters.....

      integer*4 ncols,nrows
      integer file_type
      character*50 fin,deffin
      character*30 DirName
      character*20 file_type_name(4)

      logical First,OK

C...utility storage.....

      integer*2 ncols2, nrows2
      integer   Choice

C...determine format of class data file.....

49    if (First) then
         Choice = 3
      else
         Choice = file_type
      end if

      write (*,50)
50    format (/, ' File type is',/)
      do i = 1,3
         write (*,55) i, file_type_name(i)
55       format ( '   (',i1,') - ',a20)
      end do

      call ENTINT (' Enter choice ',.true.,Choice,Choice,OK)

      if (((Choice.lt.1).or.(Choice.gt.3)).or.(.not.OK)) then
         print *,'*** INVALID CHOICE ***'
         goto 49
      else
         file_type = Choice
      end if

C.....we are skipping RNL format anyway.....

      if (file_type.eq.4) then
         print *,' '
         print *,'*** WARNING *** This type not debugged yet.'
         print *,' '
      end if

C.....get the name of the input data file.....

51    print *,' '
      if (First) then
         deffin = '                                '
      else
         deffin = fin( index(fin,']')+1 : lchar(fin) )
      end if

      write  (*,52) deffin
52    format (' Full name of data file ["',
     +            a<lchar(deffin)>,'"]:',$)
      read   (*,'(a32)') fin

      call convert_filename (fin, deffin, DirName)

      open   (unit=1,name=fin,status='old',form='unformatted',
     +          err=53)
      go to 54
53    print *,'**ERROR** Cannot open file ',fin
      go to 51
54    continue

      call Get_File_Size (1,file_type,nrows,ncols,first)

      print *,' '
      print 44,ncols,nrows
44    format(' File has ',i5,' Columns and ',i5,' Rows.')
     
      return
      end

C *****************************************************************
C    SUBROUTINE : Get_Origin
C
C    PURPOSE: This routine gets the orgin coordinates from the user.
C             The origin must lie on the positive axes.
C
C    INPUT PARAMETERS:
C             largest_x - the largest x-value of any previous plot
C             largest_y - the largest y-value of any previous plot
C             first     - true if this is the first plot
C             xsize,
C             ysize     - length, width of the plot in mm
C             plotter   - device on which plot to be made
C             lastplotter - previously used plotting device
C             WantsRegs - true if registration marks to be plotted
C             UsageArray - shows area of plotting surface covered
C
C    OUTPUT PARAMETERS:
C             xorig - the origin in the x direction in mm
C             yorig - the origin in the y direction in mm
C             UsageArray - shows area of plotting surface covered
C
C ****************************************************************

      subroutine Get_Origin (xorig,yorig,xsize,ysize,first,
     +                       largest_x,largest_y,UsageArray,plotter,
     +                       lastplotter,WantsRegs)

      real*4 xorig,yorig,largest_x,largest_y
      character*60 UsageArray(100)  ! 1 character shows approx. 30x30 mm
      logical first,WantsRegs,OK
      integer plotter,LastPLotter
      character key

C................ begin subroutine get_origin ......................

      if ((First).or.(lastplotter.ne.plotter)) then
         do i = 1,100
            UsageArray(i) = '                              '
         end do
      end if

344   continue

      if ((plotter.eq.1).or.(plotter.eq.2)) then     ! Applicon or Versatec

         if ((.not.First).and.(lastplotter.eq.plotter)) then

            print *,' '
            print *,'Hit <Enter> to see approximately how much paper ',
     +              'covered so far:'
            read (*,'(a1)') key

            call Display_Vers_Usage (UsageArray,largest_x)

            write (*,10) largest_x,largest_y
10          format (' Largest x value so far = ',f9.4,
     +              '  Largest y value so far = ',f9.4)
      
         else

            print *,' '            
            print *,' (0,0) y ->  860 mm'
            print *,'   *-----------+'
            print *,' x |           |'
            print *,' | |           |'
            print *,' V |           |'
            print *,'   :           :'
            print *,' '

         end if

      else if (plotter.eq.3) then     ! Optronics

         if ((.not.First).and.(lastplotter.eq.plotter)) then

            print *,' '
            print *,'Hit <Enter> to see approximately how much ',
     +              'plotting surface covered so far:'
            read (*,'(a1)') key

            call Display_Opt_Usage (UsageArray)

            write (*,10) largest_x,largest_y

         else      

            print *,' '
            print *,'     (0,0)      y ->   1000 mm'
            print *,'        *----------------+'
            print *,'   x    |                |'
            print *,'   |    |                |'
            print *,'   V    |                |'
            print *,'        |                |'
            print *,'        |                |'
            print *,'1000 mm *----------------+'
            print *,' '

         end if

      else if (plotter.eq.4) then ! Tektronix screen

         print *,' '
         print *,' 270 mm +------------+'
         print *,'        |            |'
         print *,'      ^ |            |'
         print *,'      y |            |'
         print *,'        *------------+'
         print *,'      (0,0)  x ->   360 mm'
         print *,' '
         print *,'Remember - plot will be sideways.'

      end if

      if (WantsRegs) then
         write (*,11) (xsize + 20),(ySize + 20)
      else
         write (*,11) xsize,ysize
      end if
11    format (' Size of this plot will be X: ',f9.4, '  Y: ',f9.4,/,
     +        ' (Allow extra room for legend, titles, comments)')

350   defx = 0.0
      defy = 0.0
      print *,' '
360   call ENTREAL (' Enter the X-origin in mm ',.true.,defx,xorig,OK)
         if (.not.OK) goto 360
370   call ENTREAL (' Enter the Y-origin in mm',.true.,defy,yorig,OK)
         if (.not.OK) goto 370

C.....if the user left no room on paper.....
C.....for reg. marks, make room and give warning.....

      Moved = .false.

      if ((xorig.lt.0.0).and.(plotter.ne.4))
     +then
         print *,' '
         print *,'*** WARNING *** X-origin cannot be less than 0.0'
         print *,' '
         xorig = 0.0
      end if

      if ((yorig.lt.0.0).and.(plotter.ne.4))
     +then
         print *,' '
         print *,'*** WARNING *** Y-origin cannot be less than 0.0'
         print *,' '
         yorig = 0.0
      end if

      if ((plotter.eq.1).or.(plotter.eq.2)) then
         if ((yorig+ysize).gt.860) then
            print *,' '
            print *,'*** WARNING *** Plot too wide. May be printed on ',
     +              'separate sheets.'
            print *,' '
         end if
      else if (plotter.eq.3) then
         if (((yorig+ysize).gt.1000).or.((xorig+xsize).gt.1000)) then
            print *,' '
            print *,'*** WARNING *** Plot too large. May be ',
     +              'printed on separate sheets.'
            print *,' '
         end if
      end if

      if (plotter.ne.4) then

C...........create temporary display showing area covered by plot.....
C...........overlaps shown as '*', new area shown as '.'.....

         overlap = .false.
         do ix = jmax0(jint(xorig/30),1),
     +               jmin0(jint((xorig+xsize)/30),30)
            do iy = jmax0(jint(yorig/15),1),
     +                 jmin0(jint((yorig+ysize)/15),60)
               if (UsageArray(ix)(iy:iy).eq.'X') then
                  UsageArray(ix)(iy:iy) = '*'
               else
                  UsageArray(ix)(iy:iy) = '.'
               end if
            end do
         end do

         if (plotter.eq.2) then
            print *,' '
            print *,'Hit <Enter> to see approximately how much paper ',
     +              'this plot will cover:'
            read (*,'(a1)') key

            call Display_Vers_Usage 
     +                 (UsageArray,AMAX1(largest_x,xorig+xsize))

         else if (plotter.eq.3) then

            print *,' '
            print *,'Hit <Enter> to see approximately how much paper ',
     +              'this plot will cover:'
            read (*,'(a1)') key

            call Display_Opt_Usage (UsageArray)

         end if

      end if

C.....confirm with user whether origin OK.....

100   print *,'Origin is ', xorig, yorig
      write (*,102)
102   format(/,' Is this OK? (Y/N) [Y]: ',$)
      read(*,'(a1)',err=100) key

      if ((lchar(key).ne.0).and.
     +    ((key.ne.'y').and.(key.ne.'Y')))
     +then

C........if not OK, restore the display showing area covered.....

         do ix = jmax0(jint(xorig/30),1),
     +                     jmin0(jint((xorig+xsize)/30),30)
            do iy = jmax0(jint(yorig/15),1),
     +                 jmin0(jint((yorig+ysize)/15),60)
               if (UsageArray(ix)(iy:iy).eq.'*') then
                  UsageArray(ix)(iy:iy) = 'X'
               else if (UsageArray(ix)(iy:iy).eq.'.') then
                  UsageArray(ix)(iy:iy) = ' '
               end if
            end do
         end do

C........go back and get origin again.....
         
         goto 344
      end if

C.....origin is OK.....

C.....update to show approx. the location of image on paper.....
C.....permanently, with all '.'s changed to 'X's.....
C.....leave in '*'s to show overlaps.....

      do ix = jmax0(jint(xorig/30),1),jmin0(jint((xorig+xsize)/30),30)
         do iy = jmax0(jint(yorig/15),1),
     +              jmin0(jint((yorig+ysize)/15),60)
            if (UsageArray(ix)(iy:iy).eq.'.') then
               UsageArray(ix)(iy:iy) = 'X'
            end if
         end do
      end do

C.....if there are to be reg. marks, move the origin to.....
C.....allow them room.....

      if (WantsRegs) then
         xorig = xorig + 10.0
         yorig = yorig + 10.0
      end if

      print *,' '
      print *,'Press ENTER to continue'
      read(*,'(a1)') key

      return
      end

C *****************************************************************
C    SUBROUTINE : Display_Vers_Usage
C
C    PURPOSE: Displays approx. amount of paper covered for
C             Versatec plot.
C
C    INPUT PARAMETERS:
C             UsageArray - shows plotting surface
C             largest_x  - largest x-value used on plotting 
C                          surface
C
C ****************************************************************

      subroutine Display_Vers_Usage (UsageArray,largest_x)

      character*60 UsageArray(100)  ! 1 character shows approx. 30x30 mm
      real*4 largest_x

      print *,'(0,0)      y ->              ',
     +        '                               860 mm'
      print *,'*----------------------------',
     +        '--------------------------------+'

      iend = JMIN0(jint(largest_x/30)+1, 100)
      iend = JMAX0(iend,15)

      write (*,10) (UsageArray(i), i = 1,iend)
10    format (' |', a60, '|')
      print *,':                            ',
     +        '                                :'
       print *,' x  y ->'
       print *,' V'

       return
       end

C ********************************************************************
C    SUBROUTINE : Display_Opt_Usage
C
C    PURPOSE: Display approx. how much area covered by Optronics plot
C
C    INPUT PARAMETERS:
C             UsageArray - shows plotting surface
C
C ********************************************************************


      subroutine Display_Opt_Usage (UsageArray)

      character*60 UsageArray(100)  ! 1 character shows approx. 30x30 mm
      real*4 largest_x

      print *,'  (0,0)      y ->              ',
     +        '                                  1000 mm'
      print *,'   *----------------------------',
     +        '--------------------------------+'
      iend = 30
      write (*,10) (UsageArray(i), i = 1,iend)
10    format ('    |', a60, '|')
      print *,'   *----------------------------',
     +        '--------------------------------+'
      print *,'1000 mm'
      print *,' x  y ->'
      print *,' V'

      return
      end

C *****************************************************************
C    SUBROUTINE : Set_Up_Channels_Query
C
C    PURPOSE: This routine sets up GIMAGE to read in the 3 input
C             channels
C
C    OUTPUT PARAMETERS:
C             rmax - color intensity range
C
C ****************************************************************

      subroutine Set_Up_Channels_Query (rmax)

      real*4 rmax
      logical OK

33    if (rmax.eq.255.0) then
         kk = 2
      else
         kk = 1
      end if

      write (*,50)
50    format (/,' Will input channel data values be',/,
     +          '    (1) 0 - 16',/,
     +          '    (2) 0 - 255',/)
      call ENTINT (' Enter choice (1 or 2) ',.true.,kk,kk,OK)

      if (((kk.ne.1).and.(kk.ne.2)).or.(.not.OK)) then
         print *,'*** INVALID *** You must select 1 or 2'
         goto 33
      end if

      if (kk.eq.1) then
           rmax = 16.
      else if (kk.eq.2) then
           rmax = 255.
      endif

      return
      end

C *****************************************************************
C    SUBROUTINE : Get_Values
C
C    PURPOSE: This routine allows the user to change the size of
C             the plot.
C
C    INPUT PARAMETERS:
C             dec     - decimation factor *** NOT USED
C             plotter - the choosen plotter 
C             PlotterName - names of all plotters
C             first   - true if this is the first plot
C
C    INPUT/OUTPUT PARAMETERS:
C             plpix   - image output grid size in mm
C
C    OUTPUT PARAMETERS:
C             nrows   - the number of scan lines in the image
C             ncols   - the number of pixels per line
C             ixsiz   - the num. of pixels per line in the output image
C             iysiz   - the num. of lines in the output image
C             scale   - scaling factor
C             first   - true if first plot on paper
C
C ****************************************************************

      subroutine Get_Values (plotter,plpix,nrows,ncols,ixsiz,iysiz,
     +                        xsize,ysize,dec,scale,first,plotterName)
  
      integer plotter
      real*4 plpix,new_plpix,dec,scale
      integer*4 nrows,ncols,ixsiz,iysiz
      logical*1 right,first
      character*1 key
      character*40 PlotterName(5)
      real*4 xsize,ysize
      integer v

      include '[bcvax.fort]txtfnct.cns'

      right = .FALSE.
      DO WHILE (.not.right)
         new_plpix = plpix * scale

         iysiz = jnint(ncols * dec) ! compute number of decimated/
         ixsiz = jnint(nrows * dec) ! replicated pixels per row/column to plot

         xsize = ixsiz * new_plpix  ! calculate size in mm of final plot
         ysize = iysiz * new_plpix 

         write(*,*) SBLACK,CLR,' '
         write(*,*) DBLW,UL,'Plot Size',RST
         print *,' '
         print *,' '
         print *,'Selected Plotter: ', PlotterName(plotter)
         print *,' '
         if ((plotter.eq.1).or.(plotter.eq.2)) then
            print *,'Paper width             = 860 mm'
         else if (plotter.eq.3) then
            print *,'Max. plot size          = 1 meter x 1 meter'
         else if (plotter.eq.4) then
            print *,'Screen size (approx.)   = 350 x 260 mm'
            print *,'(Plot will be sideways)' 
         end if
         print *,' '
         print *,'Basic plotter grid size = ', plpix, ' mm'
         print *,'Scaling factor          = ', scale
         print *,'Plotted pixel size      = ', new_plpix, ' mm'
         print *,'Number of columns = ',iysiz,
     +           ' Number of rows = ',   ixsiz
         print *,' '
         print *, 'Plot width in mm : ',ysize,
     +            'Length in mm: ',     xsize

         if ((plotter.eq.1).or.(plotter.eq.2))
     +   then
            if (ysize.gt.860) then
               print *,'*** WARNING *** Plot may be too wide ',
     +                 'for paper.'
            end if
         else if (plotter.eq.3) then
            if ((xsize.gt.1000).or.(ysize.gt.1000)) then
               print *,'*** WARNING *** Plot may be too large ',
     +                 'for plotting surface.'
            end if
         else if (plotter.eq.4) then
            if ((ysize.gt.360).or.(xsize.gt.270)) then
               print *,'*** WARNING *** Plot may be too big ',
     +                 'for screen.'
            end if
         end if
3        print 4 
4        format (/,' Would you like to change the size ',
     +             'of the plot (Y/N)? : ',$)
         read (*,'(a1)') key

         if ((key.eq.'Y').or.(key.eq.'y')) then
            call Get_Scale (scale)
         else if ((key.eq.'N').or.(key.eq.'n')) then
            right = .true.
         else 
            print *,'*** INVALID RESPONSE ***'
            goto 3
         end if

      ENDDO

      plpix = new_plpix

      return
      end

C *****************************************************************
C    SUBROUTINE : Get_Plotter
C
C    PURPOSE: This routine gets the plotter type and output grid
C             size from the user. The user picks what is wanted
C             from a menu.
C
C    INPUT PARAMETERS:
C             first - true if this is the first plot
C
C    OUTPUT PARAMETERS:
C             plotter     - the choosen plotter
C             lastplotter - one used for previous plot
C             plpix       - image output grid size in mm
C
C ****************************************************************

      subroutine Get_Plotter(plotter,lastplotter,plpix,first)

      integer plotter
      real*4 plpix
      logical first,OK
      character key*1

12    print *,' '
      write  (*,11)
11    format (
     + '     Plotter                          Grid Size (mm)',//,
     + ' (1) APPLICON                           0.2031',/,
     + ' (2) VERSATEC                           0.2540',/,
     + ' (3) OPTRONICS                          0.2540',/,
     + ' (4) TEKTRONIX 41** (screen plot)       0.8124',/,
     + ' (5) Select from GROUTE        [default 0.2540]',/)
      call ENTINT (' ENTER choice (1-5) ',.true.,plotter,plotter,OK)

      if ((plotter.lt.1.or.plotter.gt.5).or.(.not.OK)) then
        print *,'**ERROR** Invalid choice!'
        go to 12
      endif

      if (plotter.eq.1) then
         plpix = 0.2031
         print *,' '
         print *,'*** WARNING *** Program has not been tested on ',
     +           'Applicon plotter.'
         print *,' '
         print *,'Press ENTER to continue'
         read(*,'(a1)') key
      else if ((plotter.eq.2).or.(plotter.eq.3)) then
         plpix = 0.2540
      else if (plotter.eq.4) then
         plpix = 0.8124
      else if (plotter.eq.5) then
         plpix = 0.2540
      end if

      if (((plotter.ne.lastplotter).and.(.not.first))
     +   .and.(lastplotter.ne.4))
     +then
15       write (*,20)
20       format 
     +    (/,' *** WARNING *** This will close the previous ',
     +       'plot file permamently.',/,
     +       ' Are you sure you want to change the plotter (Y/N)?: ',$)
         read (*,'(a1)',err=15) key
         if ((key.eq.'Y').or.(key.eq.'y')) then
            return
         else 
            goto 12
         end if
      end if

      return
      end

C *****************************************************************
C    SUBROUTINE : Get_Decimation   *** NOT USED
C
C    PURPOSE: First finds out the scaling desired by user (same,
C             bigger, or smaller); then, gets the factor by which
C             image is to be enlarged or reduced (decimated).
C
C    OUTPUT PARAMETERS:
C             Dec - decimation factor
C ****************************************************************

      subroutine Get_Decimation (dec)

      real*4 Dec
      integer*2 Factor,Which
      character More

1     continue
      print *, ' ' 
      print *, 'DECIMATION / REPLICATION'
      write  (*,10)
10    format ('    (1) Output each pixel once',/,
     +        '    (2) Decimate pixels',/,
     +        '    (3) Replicate pixels',/,
     +        ' Enter choice: ',$)
      read   (*,*,err=1,iostat=ierr) Which
         if (((Which.ne.1).and.(Which.ne.2)).and.(Which.ne.3)) then
            goto 1
         end if

      if (Which.eq.1) then
         Dec = 1.0
      else if (Which.eq.2) then
11       write  (*,12)
12       format (' Plot every Kth pixel, k = ',$)
         read   (*,*,err=11,iostat=ierr) Factor
         if (Factor.lt.2) then
            print *,'**ERROR** k cannot be less than 2'
            go to 11
         endif
         Dec = Factor
         Dec = 1/Dec
      else
13       write  (*,14)
14       format (' Plot each pixel k times, k = ',$)
         read   (*,*,err=13,iostat=ierr) Factor
         if (Factor.lt.2) then
            print *,'**ERROR** k cannot be less than 2'
            go to 13
         endif
         Dec = Factor
      end if

      return
      end


C *****************************************************************
C    SUBROUTINE : Get_Image_Pixel_Size   *** NOT USED
C
C    PURPOSE: This routine gets the image pixel size from the user.
C             pixsz > 0
C
C    OUTPUT PARAMETERS:
C             pixsz - output image pixel size in mm
C
C ****************************************************************

      subroutine Get_Image_Pixel_Size(pixsz)

      real*4 pixsz

1     continue
      write  (*,12)
 12   format (' Enter Image Pixel Size (i pix = X meters), X = ',$)
      read   (*,*,err=1,iostat=ierr) pixsz
      if (pixsz.le.0.) then
        print *,'**ERROR** Pixel size must be greater than 0'
        go to 1
      endif

      return
      end

C *****************************************************************
C    SUBROUTINE : Get_Scale
C
C    PURPOSE: This routine gets the scale from the user.
C             scale > 0
C
C    OUTPUT PARAMETERS:
C             scale - the scaling factor
C
C ****************************************************************

      subroutine Get_Scale(scale)

      real*4 scale

 1    continue
      write  (*,14)
 14   format (' ',/,
     +        ' SCALING FACTOR',/,
     +        ' eg. a value of 0.5 will reduce the pixel',/,
     +        ' size and plot width/length to half of what they',/,
     +        ' would be if 1 input pixel equalled 1 plotter pixel.',
     +        //,'    Enter image scaling factor: ',$)
      read   (*,*,err=10,iostat=ierr) scale
      goto 20
10    print *,'*** ERROR *** Invalid input'
      goto 1

20    if (scale.le.0.0) then
        print *,'**ERROR** scale value must be greater than 0!'
        go to 1
      endif

      return
      end

C *****************************************************************
C    SUBROUTINE : Set_Up_Color_Table_Queries
C
C    PURPOSE: Gets color/shading information
C
C    INPUT PARAMETERS:
C             patterns - true if B&W colour scheme used
C	      Is_RGB   - true if RGB to be used; false if CMY
C             fin      - name of input data file
C             First    - true if first plot
C
C    OUTPUT PARAMETERS:
C             TextColor - A class value which will show up as black.
C			  This must be found because SPANS always
C 			  gives text pixels a class value of 255,
C			  but GIMAGE will only accept up to 98 color
C			  classes.
C             BGColor  - A class value which will show up as white.
C                        Pixel values of 0 (background) will be changed 
C                        to point to this color class value.
C             ic,im,iy - color table entry values
C             zclass   - (see comments in code in Set_Up_Color_Table)
C             kolor    -  "     "       "      "        "        " 
C             NewTable - true if new color table to be loaded; false
C                        if standard palette to be used
C             BicName  - name of BIC file
C             palName  - name of color palette file
C             BWPal    - name of B & W palette file
C             DirName - default directory for data files
C
C ****************************************************************

      subroutine Set_Up_Color_Table_Queries 
     +                       (patterns,ic,im,iy,
     +                        zclass,kolor,TextColor,BGColor,
     +                        Is_RGB,NewTable,First,PalChoice,
     +                        fin,PalName,BicName,BWpal,DirName)

      integer ic(99),im(99),iy(99)
      integer*2 TextColor,BGColor
      integer   PalChoice

      character*50 BicName,PalName,BWpal,fin,deffin
      character*30 DirName

      logical patterns, OK,Is_RGB,NewTable,First
      character More

      dimension kolor(99),zclass(98)

      data BicUnit /100/
      data PalUnit /101/

      data MaxColors /96/      ! see GIMAGE User's Manual, p.69

C.......For color palettes.....................................................

        if (.not.patterns) then

6	   print 7
7          format (
     +            ' (1) Palette is in .BIC file',/,
     +            ' (2) Palette is in .PAL file',/)
C     +            ' (3) Use standard palette',/)       ! not debugged
           call ENTINT (' Enter Choice (1-2) ',.true.,
     +                  PalChoice,PalChoice,OK)

           if (((PalChoice.lt.1).or.(PalChoice.gt.2))
     +        .or.(.not.OK)) 
     +     then
              print *,'*** INVALID *** You must select 1 or 2'
              goto 6
           end if

           if (PalChoice.eq.3) then       ! ie. use standard UNIRAS palette
              NewTable = .false.          ! (not debugged)
              return
           else
              NewTable = .true.
           end if

C..........palette must retrieved from .BIC file.....

	   if (PalChoice.eq.1) then

C.............default is <data file name>.BIC .....

8             deffin = fin( index(fin,']')+1 : lchar(fin) )     ! strip dir.
              deffin = deffin( 1 : index(deffin,'.') ) // 'BIC' ! add extension

              write (*,9) deffin
9	      format (/,' Enter full name of .BIC file ',
     +                  '(including extension) ["',
     +                  a<lchar(deffin)>,'"]: ',$)
	      read (*,'(a32)') BicName

              call convert_filename (BicName, deffin, DirName)

C.............determine name of new palette file.....

              deffin = BicName ( index(BicName,']')+1 
     +                            : lchar(BicName) )
              deffin = deffin ( 1 : index(deffin,'.') ) // 'PAL'

              write (*,710) deffin
710           format (/,' Enter full name for new palette file ',
     +                  '(incl. extension) ',
     +                  '["',a<lchar(deffin)>,'"]: ',$)
              read (*,'(a32)') PalName

              call convert_filename (PalName, deffin, DirName)

C.............decode the palette.....

	      call decode_BIC (OK, BicName, PalName, MaxColors)  

              if (OK) then
	         print *, 'Palette file ', PalName, ' has been created.'
	      else
	         print *, '*** Error *** Unable to create ', PalName
                 print *, '              from ',BicName
                 go to 6
	      end if

C...if there is already a palette file, get the name.....

           else if (PalChoice.eq.2) then

10            deffin = fin( index(fin,']')+1 : lchar(fin) )     ! strip dir.
              deffin = deffin( 1 : index(deffin,'.') ) // 'PAL' ! add extension

              write (*,11) deffin
11            format(' ENTER name of palette file (incl. extension) ["',
     +               a<lchar(deffin)>, '"]: ',$)
              read(*,'(a32)',err=10,iostat=ierr) PalName

              call convert_filename (PalName, deffin, DirName)

	   end if

C..........open & read the .PAL file......................................

           if ((PalChoice.eq.1).or.(PalChoice.eq.2)) then
              call Read_Palette (OK,Is_RGB,PalName,zclass,kolor,
     +                           ic,im,iy,TextColor,BGColor,MaxColors)
              if (.not.OK) then
                 goto 6
              end if
           end if

C..........................................................................
C..........for B&W palettes (patterns is TRUE).............................

        else

108        NewTable = .true.

           deffin = fin( index(fin,']')+1 : lchar(fin) )     ! strip dir.
           deffin = deffin( 1 : index(deffin,'.') ) // 'PAL' ! add extension

           write (*,109) deffin
109	   format (' Full name of .PAL file (including extension) ["',
     +             a<lchar(deffin)>,'"]: ',$)
	   read (*,'(a32)') BWpal

           call convert_filename (BWPal, deffin, DirName)

           print *,' '
           print *,'Looking for file ', BWpal
           print *,' '

           open(unit=40,file=BWpal,form='formatted',access='sequential',
     +          status='old',err=110)
              goto 111
110           print *, 'ERROR: File not found: ', BWpal
              goto 108
111           continue

           do ientries = 1,300
             read(40,*,end=90) j,kolor(ientries)
           enddo
90         continue
           close(40)


           do i = 1,ientries - 1	
              zclass(i) = float(i - 1)
           end do

        end if

        print *,' '
        print *,'<ENTER> to continue...'
        read (*,'(a1)') More

        return
        end

C *****************************************************************
C    SUBROUTINE : Color_Query
C
C    PURPOSE: Finds out from the user if a black & white pattern or 
C             a color plot is wanted. If color, finds out if RGB or CMY
C             scheme to be used.
C
C    INPUT PARAMETERS:
C             nlook - ??? NOT USED
C             data_type - 1: channel data; 2: color class data
C             first - true if this is the first plot
C
C    OUTPUT PARAMETERS:
C             patterns - true if B&W; false if color
C             Is_RGB   - true if RGB; false if CMY
C
C ****************************************************************

      subroutine Color_Query(patterns,Is_RGB,nlook,data_type,First)

      logical patterns,Is_RGB,First,OK
      integer*4 nlook
      integer*2 key
      integer data_type

C..............................................................................

C..............................................................................
C BLACK AND WHITE NOT DEBUGGED................................................
C
C100   if ((First).or.(.not.patterns)) then
C         key = 1
C      else
C         key = 2
C      end if
C
C      write (*,102) 
C102   format(/,' (1) Colour Plot to be Produced',/,
C     +         ' (2) B/W Pattern Plot to be Produced',/)
C      call ENTINT (' ENTER Choice (1 or 2) ',.true.,key,key,OK)
C
C      if ((key.lt.1.or.key.gt.2).or.(.not.OK)) then
C         print *,'**ERROR** Invalid choice!'
C         go to 100
C      endif         
C
C      if (key.eq.2) then
C         print *,' '
C         print *,'*** WARNING *** Not working for black and white.' 
C         print *,' '
C         patterns = .true.
C      end if
C
C..............................................................................

      patterns = .false.

      if (.not.Patterns) then
105      if ((First).or.(Is_RGB)) then
            key = 1
         else
            key = 2
         end if

         print *,' '
	 print 110
110	 format (' Will the color palette/channel ',
     +           'data used contain ',/,
     +           '    (1) RGB values',/,
     +           '    (2) CMY values',/)
         call ENTINT (' ENTER Choice (1 or 2) ',.true.,key,key,OK)

	 if ((key.eq.1).and.(OK)) then
            Is_RGB = .true.
	 else if ((key.eq.2).and.(OK)) then
            Is_RGB = .false.
	 else
            print *,'*** INVALID *** You must select 1 or 2'
            goto 105
         end if
      end if

      return
      end

C *****************************************************************
C    SUBROUTINE : Shorten_Query
C
C    PURPOSE: Finds out from the user if an abbreviated plot wanted
C             (eg. to save time when checking on screen before paper
C             plot).
C
C    OUTPUT PARAMETERS:
C             Shorten  - true if plot to be shortened
C             ShortNum - number of lines to actually plot
C
C ****************************************************************

      subroutine Shorten_Query (Shorten,ShortNum)

      logical   Shorten,OK
      integer*4 ShortNum

C.................................................................

400   write (*,500)
500   format (//,' Do you want an abbreviated plot to be done? ',
     +           '(Y/N) [N]: ',$)
      read (*,'(a1)') key
      Shorten = ((key.eq.'Y').or.(key.eq.'y'))

      if (Shorten) then
600      print *,' '
         call ENTINT (' Number of lines to plot before quitting ',
     +                .true.,50,ShortNum,OK)
         
         if ((ShortNum.lt.1).or.(.not.OK)) then
            print *,'*** INVALID ***'
            goto 600
         end if
      end if

      return
      end

C*******************************************************************
C     SUBROUTINE: Convert_Filename
C
C     DESCRIPTION:
C         If the input filename is blank, adds DirName to deffin
C         to get new infin; if infin not blank but has no directory
C         specified, DirName added to front of infin; otherwise,
C         infin left alone, directory part of infin is moved into
C         DirName, replacing whatever was there before.
C
C******************************************************************
 
      subroutine Convert_filename (infin, deffin, DirName)

      character infin*50, deffin*50, DirName*30

      if (lchar(infin).eq.0) then
         infin = DirName (1:lchar(DirName)) // deffin
      else if (index(infin,']').eq.0) then
         infin = DirName (1:lchar(DirName)) // infin
      else
         DirName = infin ( 1 : index(infin,']'))
     +             // '                          '
      end if

      return
      end
